#include "vumeter.h"

vumeter::vumeter(void(*p_valcb)(int),const char* dev)
{
	int mode;

	mp_valcb=p_valcb;
	m_run=false;

	// open device for reading
	if ((m_audio_fd=open(dev,O_RDONLY,0))==-1)
		fprintf(stderr,"flrec: cannot open %s for vumeter\n",dev);
	else{

		// set mono mode
		mode=1;
		if (ioctl(m_audio_fd,SNDCTL_DSP_CHANNELS,&mode)!=0)
			fprintf(stderr,"flrec: cannot set %s for vumeter\n",dev);
		if (mode!=1) fprintf(stderr,"flrec: cannot set mono mode for vumeter\n");

		// set 8 bit audio data
		mode=AFMT_U8;
		if (ioctl(m_audio_fd,SNDCTL_DSP_SETFMT,&mode)!=0)
			fprintf(stderr,"flrec: cannot set %s for vumeter\n",dev);
		if (mode!=AFMT_U8) fprintf(stderr,"flrec: cannot set U8 data for vumeter\n");

		// set 8Khz sampling rate
		mode=8000;
		if (ioctl(m_audio_fd,SNDCTL_DSP_SPEED,&mode)!=0)
			fprintf(stderr,"flrec: cannot set %s for vumeter\n",dev);
		if (mode!=8000) fprintf(stderr,"flrec: cannot set 8Khz sampling for vumeter\n");

		// start threads
		m_run=true;

		// start threads
		pthread_create(&m_tid1,NULL,&vumeter::EntryPoint,(void*)this);
		usleep(1000);
		pthread_create(&m_tid2,NULL,&vumeter::EntryPoint,(void*)this);
	}
}

vumeter::~vumeter()
{
	if (m_run){
		m_run=false;
		pthread_join(m_tid1,NULL);
		pthread_join(m_tid2,NULL);
	}

	// close device
	if (m_audio_fd!=-1) close(m_audio_fd);
}

void* vumeter::EntryPoint(void* pthis)
{
   vumeter* pt = (vumeter*)pthis;
   return (pt->thrCycle());
}

void* vumeter::thrCycle()
{
	int len;
	unsigned char buf[AUDIO_BUF_SIZE];
	unsigned char max;

	while (m_run){
		if ((len=read(m_audio_fd,buf,AUDIO_BUF_SIZE))!=0)
			fprintf(stderr,"flrec: vumeter reading error\n");
		else{
			// analyze buffer
			max=0;
			for (;len>0;len--){
				if (*(buf+len-1)&0x80) max |= *(buf+len-1)>1;
				else max |= (*(buf+len-1)^0x7f);
			}
			mp_valcb((int)max);
		}
		usleep(1000);
	}
	
	return NULL;
}



